#!/usr/bin/perl
#
# This program uses the ps command to obtain information about
# the gse servers and the client processes running on localhost.
# This information will be displayed as a table as shown below:
# 
# GSE Server at m468a.space.umn.edu (128.101.218.50)
# 
#  -gse will display the servers that are unique to the Realtime version  
#       of the server
#
# Server               PID                  %CPU
# TmDaq                2572                 0.0                  
# TC_Handler           -                    -                    
# TMServerAgent        2573                 0.0                  
# 
# 
# Client Process       PID                  %CPU
# Client_1             2643                 0.3
# Client_2             2685                 0.1 
#
#
# This script is invoked by the command:
#                 
# gsecontrol -stat
#
#
# J. Silvis
# Dec 2004
#
# Gives hostname display
# July 2006
#
# Gives version number
# Aug 2006
#
# Gives MOC description
#
# Aug 2006
# ***********************************************************
use POSIX qw(ceil floor);
use Time::Local;
$TRUE = 1;
$FALSE = 0;

$ServerUpTime = 0;
$totNumberOfClients = 0;
$numArgs = $#ARGV + 1;
$GSE_ServersPresent = 0;
foreach $argnum (0 .. $#ARGV) {
   $ArgString = $ARGV[$argnum];
   if ($ArgString =~ /-gse/ ){
    $GSE_ServersPresent = 1; 
   }
}

use Sys::Hostname;
$updateTime = 1; # number of seconds between updates of the output
$clear = `clear`;  # cache the screen clear command
$TableFormat = "%-20s %-10s %-10s \n"; 
$FormatForClient = "%-12s %-7s %-8s %-5s %-20s\n"; 


$hostname = hostname();
$ip = `ipconfig getifaddr en0`;
chomp($ip);


$CReturns = sprintf("%s", "\n\n");    # used as a separator between tables
$ServerHeader = sprintf ($TableFormat, "Server", "PID", "Client Box",  "%CPU");
#$ClientHeader = sprintf ($FormatForClient, "Client Process", "PID", "Client Uptime", "%CPU");
$ClientHeader = sprintf ( "Client Process      PID     Time     %CPU  Remote Client\n");


while(1) {
# 
   $AgentStatus = server_ps("ServerAgent.jar","TMServerAgent" , $TableFormat);
   if ($AgentStatus =~ /\-\s+\-/) {
     $totNumberOfClients = 0; # Server has gone down
   }


# Get PIDs and CPU usage of the servers
  my $ServerStatus = "";

    $ServerStatus = $ServerStatus .  "Server Hosted at $hostname (" . $ip . ")\n";

  if ($GSE_ServersPresent) {
    # if the DAQ is up find a description of the MOC that it is 
    # connected to.
    $ServerStatus = $ServerStatus . "MOC : " . moc_description("TmDaq.jar") ;
  }
  $ServerStatus = $ServerStatus . "TMlib Version Number: " . ConfigFileParameter("ServerVersion") ;
  $ServerStatus = $ServerStatus . "Logging Level       : " . ConfigFileParameter("TM_log_level") ;

  $ServerStatus = $ServerStatus . gmtime();
  $Server_Uptime = get_uptime("ServerAgent.jar");
  $ServerStatus = $ServerStatus . ", uptime is " .  $Server_Uptime;

  $ServerStatus = $ServerStatus . $CReturns;
  $ServerStatus = $ServerStatus . $ServerHeader;

  if ($GSE_ServersPresent) {
      $ServerStatus = $ServerStatus . server_ps("TmDaq.jar","TmDaq" , $TableFormat);
      $ServerStatus = $ServerStatus . server_ps("TC_Handler.jar","TC_Handler" , $TableFormat);
      $ServerStatus = $ServerStatus . server_ps("Record.jar","Recorder" , $TableFormat);
  }
  
  $ServerStatus = $ServerStatus . $AgentStatus;
  
  $ServerStatus = $ServerStatus . server_ps("CentralLogger.jar","CentralLogger" , $TableFormat);
  $ServerStatus = $ServerStatus . 
       server_ps("(apache.*TMlib_WebApps.*start)|(Djetty.*)|TMlib_WebApps","WebApps" , $TableFormat);
#
# Get PIDs and CPU usage of the clients
  $ClientStatus = client_ps("tcpPidSess","Client" , $FormatForClient,$num,$totNumberOfClients);
  print $clear;
  print   $ServerStatus .
         $CReturns . "Number of clients: $num current, $totNumberOfClients total\n" .
         $ClientHeader . $ClientStatus;
  sleep $updateTime;
}
# End of the main part of the script


sub ConfigFileParameter( ){
#
# This subroutine will find the version number of the 
# of the current server.  This information is stored in the 
# Info.plist file.
#
 $ParameterName =$_[0];
$VersionNum = 0;
open(inFile, "../config/Info.plist");
while( $line = <inFile> ) {
   if ($line =~ /(.*)$ParameterName(.*)/) {
     $line = <inFile>;
     $line =~ s/(.*>)(\w+)(<.*)/\2/;
     $ParameterValue = $line;
     break;
   }
}
  return $ParameterValue;
}
#
# This opens a short file that contains the properties of the 
# MOC that the DAQ is currently connected to.  This file is 
# written by TmDaq.java.  
sub moc_description( ){
  
  
  $ServerName =$_[0]; # the actual process name as seen by PS
  $ServerFound = 0;
  open (SERVER_PS, "ps -a -x -ww  -o  pid,%cpu,command |" );
  while(<SERVER_PS>){
    if (/$ServerName/) {
      $ServerFound = 1;
    }
  }
  $MOC_DESCRIPTION = "-\n";
   # Only look for a MOC description if a DAQ is up. 
  if($ServerFound) {
  open(inFile, "../config/CurrentMocProperties.xml");
  while( $line = <inFile> ) {
    if ($line =~ /^(.*)DESCRIPTION(.*)\n$/) {
      $line =~ s/.*\"(.*)\".*/\1/;
      $MOC_DESCRIPTION=$line;
      break;
    }
  }
}
  return $MOC_DESCRIPTION;
}




sub server_ps( ){
$ServerName =$_[0]; # the actual process name as seen by PS
$PrintName = $_[1]; # name that will be displayed by this program
$Format = $_[2];    # format for the output
#

$ServerFound = 0;
$MyServerLineOutput = "";
open (SERVER_PS, "ps -a -x -ww  -o  pid,%cpu,command |" );
while(<SERVER_PS>){
   if (/$ServerName/) {
     $ps_output_scalar = $_;
      $ps_output_scalar =~ s/^\s*(.*?)\s*$/$1/;
      @ps_output = split(/\s+/,$ps_output_scalar );
      $MyServerLineOutput = $MyServerLineOutput . 
                  sprintf ($Format, $PrintName, $ps_output[0],  $ps_output[1]);
      $ServerFound = 1;
   }
}

if ($ServerFound == 0) {
   $blank="-";
   $MyServerLineOutput = $MyServerLineOutput . 
                     sprintf ($Format, $PrintName, $blank, ,  $blank);
}

close(SERVER_PS);

return $MyServerLineOutput;
}






sub client_ps( ){
#
# When a client process is created the server agent gives it a number as 
# part of its name.  These numbers count up from 0. For instance 
# a client process as displayed by ps would be:
# silvis  2685   0.0  2.3   273904  23724  p1  S    11:06AM   6:48.98 /usr/bin/java -jar tcpPidSess.jar 8001 2 &
#
# The number "2" is the server agent ID for this client process. 
# This number is extracted from the ps output and used in the naming 
# of the clients in the list.  The client output is sorted by this value before
# printing it to the screen.
#
# ******************************************************************************
$ClientName =$_[0]; # the actual process name as seen by PS
$PrintName = $_[1]; # name that will be displayed by this program
$Format = $_[2];     # format for the output
$num = $_[3];
$totNumberOfClients  = $_[4];
#
$num =0;
$ClientFound = 0;
open (CLIENT_PS, "ps -a -x -ww  -o  pid,%cpu,command |" );
my @ClientList ;
while(<CLIENT_PS>){
   if (/$ClientName/) {
     $num++;
     $ps_output_scalar = $_;
     $ps_output_scalar =~ s/^\s*(.*?)\s*$/$1/;
     @ps_output = split(/\s+/,$ps_output_scalar );
     $Agent_id =  $ps_output[6];
     if($Agent_id > $totNumberOfClients){
       $totNumberOfClients = $Agent_id;
     }
     $Agent_length = length($Agent_id);
     while ($Agent_length < 4) {
        $Agent_id = "0" . $Agent_id;
        $Agent_length = length($Agent_id); 
     }
     $OutName = $PrintName . "_" .  $Agent_id; # The server agent ID number
     $ClientPID = $ps_output[0];
     $pid_length = length(ClientPID);
     while ($pid_length < 5) {
      $ClientPID = " " . $ClientPID;
      $pid_length = length($ClientPID); 
     }  
     $cpu = $ps_output[1];
     $ClientHostName = $ps_output[7];

     $SearchForTimeStamp = "$ClientName.*$ps_output[5]";
     $uptime = get_uptime("$SearchForTimeStamp");
     $OneClient = sprintf ($Format, $OutName, $ClientPID, $uptime,$cpu, $ClientHostName);
     @ClientList = (@ClientList,$OneClient);
     $ClientFound = 1;
  }
}


@SortedClients = sort { $a cmp $b } @ClientList; 

my $totalPrint ;
$ClientNum=1;
foreach $array_element(@SortedClients){
      $Client_str = $ClientNum;
      $Client_str_length = length($Client_str);
      while ($Client_str_length < 3) {
          $Client_str = "0" . $Client_str;
          $Client_str_length = length($Client_str);
       }
      $totalPrint = $totalPrint . $Client_str  .  "    $array_element";
      $ClientNum++;
}



if ($ClientFound == 0) {
   $blank="-";
   $totalPrint  = sprintf ( $Format,  "_",  "       _","      _","      _","      _");
}
close(CLIENT_PS);
return $totalPrint;

}

sub get_uptime(){
# This subroutine will find the length of time that a given process has been up 
# and will return that time as DAYS HR:MIN.
#
# ******************************************************************************
$ServerName =$_[0];
  %months = ( 
        Jan => 0, 
        Feb => 1, 
        Mar => 2, 
        Apr => 3, 
        May => 4, 
        Jun => 5, 
        Jul => 6, 
        Aug => 7, 
        Sep => 8, 
        Oct => 9, 
        Nov => 10, 
        Dec => 11, 
    ); 

open (SERVER_PS, "ps -a -x -ww  -o  lstart,command |" );
 $ServerFound = $FALSE;
while(<SERVER_PS>){
   if (/$ServerName/) {
     $ps_output_scalar = $_;
      @ps_output = split(/\s+/,$ps_output_scalar );
      $ServerFound = $TRUE;
   }
}
$uptime = "";
if ($ServerFound == $FALSE ){
   $uptime = "0 00:00";
}
else{
   #
   # $ps_output_scalar has the form:
   # Tue Jun 12 16:48:25 2007     java -jar ServerAgent.jar
   $StrMon = @ps_output[1];
   $clock_time = @ps_output[3];
   # Separate hours minutes and seconds
   @parts_of_clock_time = split(/:/,$clock_time );

   $year  = @ps_output[4] ;
   $mon   = $months{$StrMon};
   $mday  = @ps_output[2];
   $hours = @parts_of_clock_time[0];
   $min   = @parts_of_clock_time[1];
   $sec   = @parts_of_clock_time[2];

   $Time_Server_Sarted = timelocal($sec, $min, $hours, $mday, $mon, $year);
   $CurrentTime = time();

   $Delta_T = $CurrentTime - $Time_Server_Sarted;
   $daysup = floor(( $Delta_T/(24. * 60. * 60.))); 
   $factDay = $Delta_T/(24. * 60. * 60.) - $daysup ;

   $hrs = floor($factDay *24);
   $fracthr = ($factDay *24) - $hrs;
   $min = floor($fracthr *60);

   $strmin = "$min";
   if ($min < 10) {
     $strmin = "0" . $min;
   }
   
   $strhrs =  "$hrs";
   if ($hrs < 10) {
     $strhrs = "0" . $hrs;
   }
   
   $uptime =  $daysup . " " . $strhrs . ":" . $strmin ;
}
return $uptime;
}
